# assemble [![NPM version](https://img.shields.io/npm/v/assemble.svg)](https://www.npmjs.com/package/assemble) [![Build Status](https://img.shields.io/travis/assemble/assemble.svg)](https://travis-ci.org/assemble/assemble)

> Assemble is a powerful, extendable and easy to use static site generator for node.js. Used by thousands of projects for much more than building websites, Assemble is also used for creating themes, scaffolds, boilerplates, e-books, UI components, API documentation, blogs, gh-pages and more! Plugins for gulp and grunt are also supported.

We're happy to announce the release of Assemble v0.7.5!

## TOC

- [Install](#install)
- [Getting started](#getting-started)
  * [Running tasks](#running-tasks)
  * [API and CLI](#api-and-cli)
- [CLI](#cli)
  * [Tasks](#tasks)
  * [Options](#options)
  * [Object expansion](#object-expansion)
  * [cwd](#cwd)
  * [file](#file)
- [API](#api)
- [Templates API](#templates-api)
- [File System API](#file-system-api)
  * [.src](#src)
  * [.dest](#dest)
  * [.copy](#copy)
  * [.symlink](#symlink)
- [Task API](#task-api)
  * [.task](#task)
  * [.build](#build)
  * [.watch](#watch)
- [Release history](#release-history)
- [Test coverage](#test-coverage)
- [Contributing](#contributing)
- [About](#about)
- [Similar projects](#similar-projects)
- [Authors](#authors)
- [License](#license)

_(TOC generated by [verb](https://github.com/verbose/verb) using [markdown-toc](https://github.com/jonschlinkert/markdown-toc))_

## Install

Install with [npm](https://www.npmjs.com/):

```sh
$ npm i assemble --save
```

## Getting started

If you plan on using assemble's CLI, you'll need to use an `assemblefile.js`. Otherwise, assemble can be used like any other node.js library.

**Example**

Render a template with with [handlebars](http://www.handlebarsjs.com/):

```js
var assemble = require('assemble');
var app = assemble();

// register an engine
app.engine('hbs', require('engine-handlebars'));

// add a "page" and render it!
app.page('home.hbs', {content: 'This is {{title}}'})
  .render({title: 'Home!'}, function(err, view) {
    if (err) throw err;
    console.log(view.content);
    //=> 'This is Home!'
  });
```

### Running tasks

Create an `assemblefile.js` and add tasks to run:

```js
var assemble = require('assemble');
var htmlmin = require('gulp-htmlmin');
var app = assemble();

app.page('a.hbs', {content: '...'});
app.page('b.hbs', {content: '...'});
app.page('c.hbs', {content: '...'});

app.task('default', function(cb) {
   app.toStream('pages')
    .pipe(app.renderFile())
    .pipe(htmlmin())
    .pipe(app.dest('site'));
});

// expose your instance of assemble
module.exports = app;
```

### API and CLI

**Use assemble via CLI or API**

To run tasks, you can do one of the following:

* CLI: install Assemble globally then use the `assemble` command
* API: use the `.build` method

**CLI: Install Assemble globally**

Install `assemble` using [npm](https://www.npmjs.com/) with the following command:

```sh
$ npm i -g assemble
```

With assemble installed, you may now run `assemble` from any project that has an `assemblefile.js` in its root (the "root" of a project is wherever `package.json` is).

**API: Use the `.build` method**

_(This can be in any file, not just an `assemblefile.js`)_

```js
app.build('default', function(err) {
  if (err) throw err;
  console.log('done!');
});
```

Learn [more about tasks](#task-api).

## CLI

Run assemble from the command line.

```sh
$ assemble <tasks> [options]
```

Note that for most command line options, _order or definition makes no difference_, so tasks can be defined before or after options.

### Tasks

Optionally specify one or more tasks to run. Multiple tasks are separated by a space.

**Example**

To run tasks `foo` and `bar`, you would enter the following in the command line:

```sh
$ assemble foo bar
```

### Options

Non-task commands and options are prefixed with `--` and are specified using any of the following formats:

* single value, like `--foo`, or
* key-value pair, like `--foo=bar`. Also, key-value pairs may be separated by either `=` or a single whitespace, so `--foo=bar` and `--foo=bar` should both work.

Additionally, as mentioned above, tasks may be defined before or after options, so both of the following are equivalent:

```sh
$ assemble --cwd foo bar
# or 
$ assemble foo bar --cwd
```

**Example**

To emit views as they're loaded and log them to `stderr`, run assemble with the following command:

```sh
$ assemble --emit view
# or
$ assemble --emit=view
```

### Object expansion

Object-paths may be specified using dot-notation for **either the key or value** in a command line argument.

Additionally, assemble uses [expand-object](https://github.com/jonschlinkert/expand-object) (and some custom parsing) to make it easier to pass non-trivial options and commands via command line. So all of the following formats are possible.

**Examples**

Boolean values:

```sh
$ assemble --foo 
# { foo: true }
```

Key-value pairs:

```sh
$ assemble --foo=bar
# { foo: 'bar' }
```

Nested booleans:

```sh
$ assemble --option=foo 
# {options: { foo: true }}
```

Nested key-value pairs:

```sh
$ assemble --option=foo:bar
# {options: { foo: 'bar' }}
```

Deeply nested key-value pairs:

```sh
$ assemble --option=foo.bar.baz:qux
# {options: foo: { bar: { baz: 'qux' }}}}
```

Or on the left-side of the `=`:

```sh
$ assemble --option.foo.bar.baz=qux
# {options: foo: { bar: { baz: 'qux' }}}}
```

### cwd

Change the `cwd` for the `assemblefile.js` to run, optionally specifying any tasks to run:

```sh
$ assemble <tasks> --cwd [directory]
```

**Example**

To run the `scaffolds` example in the `examples/` directory, you would enter:

```sh
$ assemble --cwd examples/scaffolds
```

If successful, in the command line, you should see something like this:

<img width="527" alt="screen shot 2016-01-09 at 1 35 52 pm" src="https://cloud.githubusercontent.com/assets/383994/12217685/0a14294e-b6d6-11e5-9e06-dc4738f0e53a.png">

### file

The name of which file to load when running, defaulting to `assemblefile.js`:

```sh
$ assemble --file assemblefile.dev.js
```

## API

### [Assemble](index.js#L24)

Create an `assemble` app. This is the main function exported by the assemble module.

**Params**

* `options` **{Object}**: Optionally pass default options to use.

**Example**

```js
var assemble = require('assemble');
var app = assemble();
```

## Templates API

Assemble has an extensive API for working with templates and template collections. In fact, the entire API from the [templates](https://github.com/jonschlinkert/templates) library is available on Assemble.

While we work on getting the assemble docs updated with these methods you can visit [the templates library](https://github.com/jonschlinkert/templates) to learn more about the full range of features and options.

***

## File System API

Assemble offers the following low-level methods for working with the file system:

* [src](#src)
* [symlink](#symlink)
* [dest](#dest)
* [copy](#copy)

Assemble has first-class support for [vinyl-fs](http://github.com/wearefractal/vinyl-fs), so any [gulp](http://gulpjs.com) plugin can be used in your assemble pipeline.

### .src

Create a [vinyl](http://github.com/gulpjs/vinyl) stream. Takes glob patterns or filepaths to the source files to read.

**Params**

* `glob` **{String|Array}**: Glob patterns or file paths to source files.
* `options` **{Object}**: Options or locals to merge into the context and/or pass to `src` plugins

**Example**

```js
app.src('src/*.hbs');

// define `src` options
app.src('src/*.hbs', { layout: 'default' });
```

### .dest

Specify a destination for processed files.

**Params**

* `dest` **{String|Function}**: File path or rename function.
* `options` **{Object}**: Options and locals to pass to `dest` plugins

**Example**

```js
app.dest('dist/');
```

### .copy

Copy files with the given glob `patterns` to the specified `dest`.

**Params**

* `patterns` **{String|Array}**: Glob patterns of files to copy.
* `dest` **{String|Function}**: Desination directory.
* `returns` **{Stream}**: Stream, to continue processing if necessary.

**Example**

```js
app.task('assets', function() {
  // return, to let assemble know when the task has completed
  return app.copy('assets/**', 'dist/');
});
```

### .symlink

Same as `.src` but takes glob patterns or filepaths for the symlinks to read.

**Params**

* `glob` **{String|Array}**: Glob patterns or file paths

**Example**

```js
app.symlink('src/*.hbs');
```

***

## Task API

Assemble has the following methods for running tasks and controlling workflows:

* [task](#task)
* [build](#build)
* [watch](#watch)

### .task

Define a task to be run when the task is called.

**Params**

* `name` **{String}**: Task name
* `fn` **{Function}**: function that is called when the task is run.

**Example**

```js
app.task('default', function() {
  app.src('templates/*.hbs')
    .pipe(app.dest('site/'));
});
```

### .build

Run one or more tasks.

**Params**

* `tasks` **{Array|String}**: Task name or array of task names.
* `cb` **{Function}**: callback function that exposes `err`

**Example**

```js
app.build(['foo', 'bar'], function(err) {
  if (err) throw err;
  console.log('done!');
});
```

### .watch

Watch files, run one or more tasks when a watched file changes.

**Params**

* `glob` **{String|Array}**: Filepaths or glob patterns.
* `tasks` **{Array}**: Task(s) to watch.

**Example**

```js
app.task('watch', function() {
  app.watch('docs/*.md', ['docs']);
});
```

## Release history

**v0.7.0**

* Stability improvements and optimizations of the API introduced in v0.6.0.

**v0.6.0**

* Major refactor. Assemble was completely re-written from the ground-up as a standalone node.js library and is no longer a grunt plugin. Grunt plugin support has been moved to [grunt-assemble](http://assemble.io). Please see that repo for additional details.

## Test coverage

As of January 18, 2016:

```
Statements : 100% (38/38)
Branches   : 100% (8/8)
Functions  : 100% (10/10)
Lines      : 100% (38/38)
```

## Contributing

Pull requests and stars are always welcome. For bugs and feature requests, [please create an issue](https://github.com/jonschlinkert/assemble/issues/new).

If Assemble doesn't do what you need, [please let us know](https://github.com/assemble/assemble/issues/new?title=request%253A%20%3Ctitle%20here%3E)

## About

**What is Assemble?**

Assemble makes it easy to create, customize, generate and maintain complete web projects. Here are some highlights:

* Expressive, imperative API that is powerful and easy to use
* Facilitates the use of _modular, encapsulated components_ in your markup, like pages, partials and layouts, resulting in consistent design across your project
* Assemble is extremely pluggable and easy to extend with helpers, plugins, middleware or engines.
* Ability to use any data source for rendering templates, which makes it easy to begin a project using mock data and switch to a "live" data source later on.
* Use any template engine for rendering templates. You can even use multiple engines at once, Assemble will automatically detect the correct one to use on each template at render time.
* Makes it easy to transform content from markdown or any other plain text format to HTML using plugins, middleware helpers or engines.
* Assemble has full support for [gulp](http://gulpjs.com) plugins. Read our [gulp FAQ](docs/src/subjects/gulp-faq.md) for more info about gulp support.

## Similar projects

If assemble doesn't do what you need, there are some other great open source projects you might be interested in, created by our friends on GitHub:

**Static site generators**

* [docpad](https://github.com/docpad)
* [metalsmith](https://github.com/metalsmith)
* [punch](https://github.com/laktek/punch)
* [wintersmith](https://github.com/jnordberg/wintersmith)

**Blog frameworks**

* [hexojs](https://github.com/hexojs)
* [ghost](https://github.com/TryGhost/Ghost)

## Authors

**Jon Schlinkert**

* [github/jonschlinkert](https://github.com/jonschlinkert)
* [twitter/jonschlinkert](http://twitter.com/jonschlinkert)

**Brian Woodward**

* [github/doowb](https://github.com/doowb)
* [twitter/doowb](http://twitter.com/doowb)

## License

Copyright © 2016 [Jon Schlinkert](https://github.com/jonschlinkert)
Released under the MIT license.

***

_This file was generated by [verb](https://github.com/verbose/verb) on January 18, 2016._